--                                                                    --
--  procedure Units_Mapper          Copyright (c)  Dmitry A. Kazakov  --
--  Sample program for GTK+                        Luebeck            --
--  Implementation                                 Summer, 2007       --
--                                                                    --
--                                Last revision :  21:35 20 Apr 2009  --
--                                                                    --
--  This  library  is  free software; you can redistribute it and/or  --
--  modify it under the terms of the GNU General Public  License  as  --
--  published by the Free Software Foundation; either version  2  of  --
--  the License, or (at your option) any later version. This library  --
--  is distributed in the hope that it will be useful,  but  WITHOUT  --
--  ANY   WARRANTY;   without   even   the   implied   warranty   of  --
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU  --
--  General  Public  License  for  more  details.  You  should  have  --
--  received  a  copy  of  the GNU General Public License along with  --
--  this library; if not, write to  the  Free  Software  Foundation,  --
--  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.    --
--                                                                    --
--  As a special exception, if other files instantiate generics from  --
--  this unit, or you link this unit with other files to produce  an  --
--  executable, this unit does not by  itself  cause  the  resulting  --
--  executable to be covered by the GNU General Public License. This  --
--  exception  does not however invalidate any other reasons why the  --
--  executable file might be covered by the GNU Public License.       --
--____________________________________________________________________--

with Ada.IO_Exceptions;         use Ada.IO_Exceptions;
--
-- Things related to Gtk-API
--
with Gdk.Event;        use Gdk.Event;
with Gtk.GEntry;       use Gtk.GEntry;
with Gtk.Handlers;     use Gtk.Handlers;
with Gtk.Table;        use Gtk.Table;
with Gtk.Widget;       use Gtk.Widget;
with Gtk.Window;       use Gtk.Window;
with Gtk.Main;
--
-- This is what we need for unit conversions
--
with Gtk.Float_Measures_Entry;  use Gtk.Float_Measures_Entry;
with Float_Measures;	        use Float_Measures;
with Strings_Edit.Floats;       use Strings_Edit.Floats;
with Units;                     use Units;

procedure Units_Mapper is

   package Handlers is
      new Gtk.Handlers.Callback (Widget_Type => Gtk_Widget_Record);

   package Return_Handlers is
      new Gtk.Handlers.Return_Callback
          (  Widget_Type => Gtk_Widget_Record,
             Return_Type => Boolean
          );

   function Delete_Event
            (  Widget : access Gtk_Widget_Record'Class;
               Event  : Gdk_Event
            )  return Boolean is
   begin
      return False;
   end Delete_Event;

   procedure Destroy (Widget : access Gtk_Widget_Record'Class) is
   begin
      Gtk.Main.Main_Quit;
   end Destroy;

   Window     : Gtk_Window;
   Grid       : Gtk_Table;
   From_Unit  : Gtk_Unit_Entry;
   From_Value : Gtk_Entry;
   To_Unit    : Gtk_Unit_Entry;
   To_Value   : Gtk_Entry;
   Ignore     : Boolean := False;

   procedure Changed_From
             (  Widget : access Gtk_Widget_Record'Class
             )  is
   begin
      if Ignore then
         return;
      end if;
      Ignore := True;
      begin
         Set_Text
         (  To_Value,
            Image
            (  Get_Value_As
               (  Value (Get_Text (From_Value)) * Get (From_Unit),
                  Get (To_Unit)
         )  )  );
      exception
         when Data_Error =>
            Set_Text (To_Value, "Not a number");
         when End_Error | Constraint_Error =>
            Set_Text (To_Value, "");
         when Unit_Error =>
            Set_Text (To_Value, "Unit error");
      end;
      Ignore := False;
   end Changed_From;

   procedure Changed_To
             (  Widget : access Gtk_Widget_Record'Class
             )  is
   begin
      if Ignore then
         return;
      end if;
      Ignore := True;
      begin
         Set_Text
         (  From_Value,
            Image
            (  Get_Value_As
               (  Value (Get_Text (To_Value)) * Get (To_Unit),
                  Get (From_Unit)
         )  )  );
      exception
         when Data_Error =>
            Set_Text (From_Value, "Not a number");
         when End_Error | Constraint_Error =>
            Set_Text (From_Value, "");
         when Unit_Error =>
            Set_Text (From_Value, "Unit error");
      end;
      Ignore := False;
   end Changed_To;

   procedure Changed_Unit
             (  Widget : access Gtk_Widget_Record'Class
             )  is
   begin
      Set_Text (To_Unit, Get_Text (From_Unit));
      Set_Constraint (To_Unit, Get (From_Unit).SI);
      Changed_From (Widget);
   exception
      when End_Error =>
         null;
   end Changed_Unit;

begin
   --
   -- Initialization
   --
   Gtk.Main.Init;
   --
   -- Creating the main window and handle its events
   --
   Gtk.Window.Gtk_New (Window);
   Set_Title (Window, "Unit mapper (Ada GTK+)");
   Return_Handlers.Connect
   (  Window,
      "delete_event",
      Return_Handlers.To_Marshaller (Delete_Event'Access)
   );
   Handlers.Connect
   (  Window,
      "destroy",
      Handlers.To_Marshaller (Destroy'Access)
   );
   Gtk.Window.Set_Border_Width (Window, 10);
   --
   -- Creating the grid, a table to align all other widgets
   --
   Gtk_New (Grid, 2, 2, False);
   Set_Row_Spacings (Grid, 3);
   Set_Col_Spacings (Grid, 3);
   Gtk.Window.Add (Window, Grid);
   --
   -- The left column are labels
   --
   Gtk_New (From_Value);
   Attach_Defaults (Grid, From_Value, 0, 1, 0, 1);
   Handlers.Connect (From_Value, "changed", Changed_From'Access);

   Gtk_New (From_Unit);
   Attach_Defaults (Grid, From_Unit, 1, 2, 0, 1);
   Handlers.Connect (From_Unit, "changed", Changed_Unit'Access);

   Gtk_New (To_Value);
   Attach_Defaults (Grid, To_Value, 0, 1, 1, 2);
   Handlers.Connect (To_Value, "changed", Changed_To'Access);

   Gtk_New (To_Unit);
   Attach_Defaults (Grid, To_Unit, 1, 2, 1, 2);
   Handlers.Connect (To_Unit, "changed", Changed_From'Access);
   --
   -- Show everything
   --
   Show_All (Grid);
   Show (Window);
   --
   -- Enter the events processing loop
   --
   Gtk.Main.Main;
end Units_Mapper;
